Imports System.Globalization
Imports System.ComponentModel
Imports System.Resources
Imports System.Threading

Public Class Form1
    ' Apply a locale to the form and its controls.
    Private Sub ApplyLocale(ByVal locale_name As String)
        ' Make a CultureInfo and ComponentResourceManager.
        Dim culture_info As New CultureInfo(locale_name)
        Dim component_resource_manager As New ComponentResourceManager(Me.GetType)

        ' Make the thread use this locale. This doesn't change
        ' existing controls but will apply to those loaded later
        ' and to messages we get for Help About (see below).
        Thread.CurrentThread.CurrentUICulture = culture_info
        Thread.CurrentThread.CurrentCulture = culture_info

        ' Apply the locale to the form itself.
        ' Debug.WriteLine("$this")
        component_resource_manager.ApplyResources(Me, "$this", culture_info)

        ' Apply the locale to the form's controls.
        For Each ctl As Control In Me.Controls
            ApplyLocaleToControl(ctl, component_resource_manager, culture_info)
        Next ctl

        ' Perform manual localizations.
        ' These resources are stored in the Form1 resource files.
        Dim resource_manager As New ResourceManager("Localized.Form1", Me.GetType.Assembly)

        ' Tooltips.
        ToolTip1.SetToolTip(picFlag, resource_manager.GetString("picFlag.ToolTip"))

        ' ComboBox items.
        ComboBox1.Items.Clear()
        ComboBox1.Items.Add(resource_manager.GetString("ComboBox1.Items"))
        ComboBox1.Items.Add(resource_manager.GetString("ComboBox1.Items1"))
        ComboBox1.Items.Add(resource_manager.GetString("ComboBox1.Items2"))

        ' ListBox items.
        ListBox1.Items.Clear()
        ListBox1.Items.Add(resource_manager.GetString("ListBox1.Items"))
        ListBox1.Items.Add(resource_manager.GetString("ListBox1.Items1"))
        ListBox1.Items.Add(resource_manager.GetString("ListBox1.Items2"))

        ' ToolStripComboBox items.
        ToolStripComboBox1.Items.Clear()
        ToolStripComboBox1.Items.Add(resource_manager.GetString("ToolStripComboBox1.Items"))
        ToolStripComboBox1.Items.Add(resource_manager.GetString("ToolStripComboBox1.Items1"))
        ToolStripComboBox1.Items.Add(resource_manager.GetString("ToolStripComboBox1.Items2"))
    End Sub

    ' Apply the locale to a control.
    Private Sub ApplyLocaleToControl(ByVal ctl As Control, ByVal component_resource_manager As ComponentResourceManager, ByVal culture_info As CultureInfo)
        ' Debug.WriteLine(ctl.Name)
        component_resource_manager.ApplyResources(ctl, ctl.Name, culture_info)

        ' See what kind of control this is.
        If TypeOf ctl Is MenuStrip Then
            ' Apply the new locale to the MenuStrip's items.
            Dim menu_strip As MenuStrip = DirectCast(ctl, MenuStrip)
            For Each child As ToolStripMenuItem In menu_strip.Items
                ApplyLocaleToToolStripItem(child, component_resource_manager, culture_info)
            Next child
        Else
            ' Apply the new locale to the control's children.
            For Each child As Control In ctl.Controls
                ApplyLocaleToControl(child, component_resource_manager, culture_info)
            Next child
        End If
    End Sub

    ' Recursively apply the locale to a ToolStripItem.
    Private Sub ApplyLocaleToToolStripItem(ByVal item As ToolStripItem, ByVal component_resource_manager As ComponentResourceManager, ByVal culture_info As CultureInfo)
        ' Debug.WriteLine(menu_item.Name)
        component_resource_manager.ApplyResources(item, item.Name, culture_info)

        ' Apply the new locale to items contained in it.
        If TypeOf item Is ToolStripMenuItem Then
            Dim menu_item As ToolStripMenuItem = DirectCast(item, ToolStripMenuItem)
            For Each child As ToolStripItem In menu_item.DropDownItems
                ApplyLocaleToToolStripItem(child, component_resource_manager, culture_info)
            Next child
        End If
    End Sub

    ' Select English.
    Private Sub radEnglish_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radEnglish.CheckedChanged
        ' Don't do this if loading the form.
        If Not Me.Created Then Exit Sub

        If radEnglish.Checked Then ApplyLocale("en-US")
    End Sub

    ' Select German.
    Private Sub radGerman_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles radGerman.CheckedChanged
        If radGerman.Checked Then ApplyLocale("de-DE")
    End Sub

    ' Display Help About.
    Private Sub AboutToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuHelpAbout.Click
#If True Then
        ' The easy way
        MessageBox.Show(My.Resources.RuntimeStrings.Help_About)
#Else
        ' This vesion explicitly uses a ResourceManager.
        ' The base name is <ProjectName>.<FileName>.
        Dim resource_manager As New ResourceManager("Localized.RuntimeStrings", Me.GetType.Assembly)

        ' Display the message.
        MessageBox.Show(resource_manager.GetString("Help_About"))
#End If
    End Sub

    ' Exit.
    Private Sub mnuFileExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles mnuFileExit.Click
        Me.Close()
    End Sub
End Class
